/*
 * C
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "hal_lpc54608.h"
#include "fsl_power.h"

/* Fields --------------------------------------------------------------------*/

/*
 * Buffer used to store ADC converted values
 */
//static __IO uint32_t converted_values[ADC_NB_CHANNELS];
//uint8_t adc_ID = 0;
//uint32_t* array_ptr = (uint32_t*)converted_values;
/* Private functions ---------------------------------------------------------*/
void ADC_Configuration(int8_t channel_mask)
{
   /* Calibration after power up. */
    ADC_DoSelfCalibration(ADC0);

    adc_config_t adcConfigStruct;
    adc_conv_seq_config_t adcConvSeqConfigStruct;

    /* Configure the converter. */
    adcConfigStruct.clockMode = kADC_ClockSynchronousMode; /* Using sync clock source. */
    adcConfigStruct.clockDividerNumber = 1;                /* The divider for sync clock is 2. */
    adcConfigStruct.resolution = kADC_Resolution12bit;
    adcConfigStruct.enableBypassCalibration = false;
    adcConfigStruct.sampleTimeNumber = 0U;
    ADC_Init(ADC0, &adcConfigStruct);

    /* Enable channel 4's conversion in Sequence A. */
    adcConvSeqConfigStruct.channelMask =  (1U << channel_mask); 
   // adcConvSeqConfigStruct.channelMask = 240U;
    adcConvSeqConfigStruct.triggerMask = 0U;
    adcConvSeqConfigStruct.triggerPolarity = kADC_TriggerPolarityNegativeEdge;
    adcConvSeqConfigStruct.enableSingleStep = false;
    adcConvSeqConfigStruct.enableSyncBypass = false;
    adcConvSeqConfigStruct.interruptMode = kADC_InterruptForEachSequence;
    ADC_SetConvSeqAConfig(ADC0, &adcConvSeqConfigStruct);
    ADC_EnableConvSeqA(ADC0, true); /* Enable the conversion sequence A. */
}
/* API -----------------------------------------------------------------------*/



int32_t adc_manager_initialize(void)
{
   POWER_DisablePD(kPDRUNCFG_PD_VDDA);    /* Power on VDDA. */
    POWER_DisablePD(kPDRUNCFG_PD_ADC0);    /* Power on the ADC converter. */
    POWER_DisablePD(kPDRUNCFG_PD_VD2_ANA); /* Power on the analog power supply. */
    POWER_DisablePD(kPDRUNCFG_PD_VREFP);   /* Power on the reference voltage source. */
    
    
    return HAL_GPIO_OK;
    
}

int32_t adc_manager_get_value(hal_lpc54608_gpio_data* gpio)
{
   // uint32_t channel_mask;
    uint32_t channel=gpio->adc_rank;
   // printf("channel = %d \n", channel);
    adc_result_info_t adcResultInfoStruct;
    /* Configure the converter and work mode. */
       ADC_Configuration(channel);
      
       while (1)
    {
        /* Get the input from terminal and trigger the converter by software. */
        
      ADC_DoSoftwareTriggerConvSeqA(ADC0);

        /* Wait for the converter to be done. */
        while (!ADC_GetChannelConversionResult(ADC0, channel, &adcResultInfoStruct))
        {
        }
        return adcResultInfoStruct.result;
    }
}

