/* Includes ------------------------------------------------------------------*/
#include "hal_lpc54608.h"


/* Enum ----------------------------------------------------------------------*/

/*
 * List of ports used by this implementation
 * Port '0' is HAL_LPC54608_GPIO_PORT_MCU
 */
typedef enum
{
    /*
     * List of MCU ports used by this implementation
     */
    LLHAL_GPIO_PORT_0 = 1,
    LLHAL_GPIO_PORT_1 = 2,
    LLHAL_GPIO_PORT_2 = 3,
    LLHAL_GPIO_PORT_3 = 4,
    LLHAL_GPIO_PORT_4 = 5,
    LLHAL_GPIO_PORT_5 = 6,

    /*
     * List of standard ports used by this implementation
     */
    LLHAL_GPIO_PORT_ARDUINO_DIGITAL = 30,
    LLHAL_GPIO_PORT_ARDUINO_ANALOG  = 31,

    /*
     * List of specific ports used by this implementation
     */
    LLHAL_GPIO_PORT_J9  = 64,
    LLHAL_GPIO_PORT_J10 = 65,
    LLHAL_GPIO_PORT_J12 = 66,
    LLHAL_GPIO_PORT_J13 = 67,

} HAL_GPIO_IMPL_Port;
/*
 * List of ADC IDs
 */
typedef enum
{
	HAL_ADC0_ID = 0,

} hal_lpc54608_adc_IDs;

/*
 * List of ADC ranks
 */
typedef enum
{
	HAL_P016_ADC_ID = 4,
	HAL_P031_ADC_ID = 5,
	HAL_P10_ADC_ID = 6, 
	HAL_P20_ADC_ID = 7, 

} hal_lpc54608_adc_ranks;


/* Fields --------------------------------------------------------------------*/
//static adc_result_info_t adcResult0InfoStruct;

/* Public functions ----------------------------------------------------------*/

void hal_lpc54608_initialize(void)

{
    /* Enable GPIO clock for all modules. */
    CLOCK_EnableClock(kCLOCK_Gpio0);
    CLOCK_EnableClock(kCLOCK_Gpio1);
    CLOCK_EnableClock(kCLOCK_Gpio2);
    CLOCK_EnableClock(kCLOCK_Gpio3);
    CLOCK_EnableClock(kCLOCK_Gpio4);
    CLOCK_EnableClock(kCLOCK_Gpio5);

    /* Enable clock for IOCON. */
    CLOCK_EnableClock(kCLOCK_Iocon);
    CLOCK_EnableClock(kCLOCK_InputMux);
    
    /* Enable ADC clock */
   CLOCK_EnableClock(kCLOCK_Adc0);
   
    
   
    
    
}

int32_t hal_lpc54608_get_gpio_data(int32_t port, int32_t pin, hal_lpc54608_gpio_data* gpio)
{
    switch (port)
    {
    case LLHAL_GPIO_PORT_0:
    {
        /*
         * GPIO0 check and initialization
         */
        switch (pin)
        {
        case  0:
        case 16:
          gpio->adc_rank=HAL_P016_ADC_ID;
        break;
        case 23:
        case 24:
        case 25:
        case 26:
        case 27:
        case 28:
        case 31:
          gpio->adc_rank=HAL_P031_ADC_ID;
            break;
        default:	 return HAL_GPIO_NOK; /* Invalid pin or not available for external access. */
        }

        gpio->port = 0;
        gpio->pin = pin;
        break;

    }

    case LLHAL_GPIO_PORT_1:
    {
        /*
         * GPIO1 check and initialization
         */
        switch (pin)
        {
        case  0:
          gpio->adc_rank=HAL_P10_ADC_ID;
          break;
        case  1:
        case 17:
        case 18:
        case 22:

            break;
        default:
            /* Invalid pin or not available for external access. */
            return HAL_GPIO_NOK;
        }

        gpio->port = 1;
        gpio->pin = pin;
        break;
    }

    case LLHAL_GPIO_PORT_2:
    {
        /*
         * GPIO2 check and initialization
         */
        switch (pin)
        {
        case  0:
           gpio->adc_rank=HAL_P20_ADC_ID;
           break;
        case  1:
        case  2:
        case 17:
        case 18:
        case 19:
        case 20:

            break;
        default:
            /* Invalid pin or not available for external access. */
            return HAL_GPIO_NOK;
        }

        gpio->port = 2;
        gpio->pin = pin;
        break;
    }

    case LLHAL_GPIO_PORT_3:
    {
        /*
         * GPIO3 check and initialization
         */
        switch (pin)
        {
        case  0:
        case  1:
        case  2:
        case  3:
        case  4:
        case 10:
        case 11:
        case 12:
        case 14:
        case 16:
        case 17:
        case 20:
        case 21:
        case 22:
        case 23:
        case 24:
        case 26:
        case 27:
        case 28:
        case 29:
        case 30:

            break;
        default:
            /* Invalid pin or not available for external access. */
            return HAL_GPIO_NOK;
        }

        gpio->port = 3;
        gpio->pin = pin;
        break;
    }

    case LLHAL_GPIO_PORT_4:
    {
        /*
         * GPIO4 check and initialization
         */
        switch (pin)
        {
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
        case 7:

            break;
        default:
            /* Invalid pin or not available for external access. */
            return HAL_GPIO_NOK;
        }

        gpio->port = 4;
        gpio->pin = pin;
        break;
    }

    case LLHAL_GPIO_PORT_5:
    {
        /*
         * GPIO5 check and initialization
         */
        switch (pin)
        {
        default:
            /* Invalid pin or not available for external access. */
            return HAL_GPIO_NOK;
        }
    }

    case LLHAL_GPIO_PORT_ARDUINO_DIGITAL:
    {
        /*
         * ARDUINO digital port: convert port and pin using MCU port and pin
         */
        switch (pin)
        {
        // pin 0 and 1 are used by LLCOMM_UART
       // case 0:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 26, gpio);
       // case 1:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 27, gpio);
        // pin 2 is used by LCD
        //case 2:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3,  2, gpio);
        case 3:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  5, gpio);
        case 4:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 10, gpio);
        case 5:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 14, gpio);
        // pin 6 is used by LCD
        //case 6:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3,  1, gpio);
        case 7:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_1, 22, gpio);
        case 8:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  7, gpio);
        case 9:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2, 1, gpio);
        case 10:      return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3,  30, gpio);
        case 11:      return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 21, gpio);
        case 12:      return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 22, gpio);
        case 13:      return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 20, gpio);
        // pin 14 and 15 are used by I2C
        //case 14:      return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 23, gpio);
        //case 15:      return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 24, gpio);
        default:      return HAL_GPIO_NOK; /* Invalid pin or not available for external access. */
        }
    }

    case LLHAL_GPIO_PORT_ARDUINO_ANALOG:
    {
        /*
         * ARDUINO analog port: convert port and pin using MCU port and pin
         */
        switch (pin)
        {
        case 0:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 16, gpio);
        case 1:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 31, gpio);
        case 2:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_1,  0, gpio);
        case 3:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2,  0, gpio);
       // case 4:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3,  4, gpio);
      //  case 5:       return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_1, 1, gpio);
        default:       return HAL_GPIO_NOK; /* Invalid pin or not available for external access. */
        }
    }

    case LLHAL_GPIO_PORT_J9:
    {
        /*
         * J9 port: convert port and pin using MCU port and pin
         */
        switch (pin)
        {

        //case  2:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_1, 18, gpio); //
        //case  4:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_1, 17, gpio); //CAN1 TD
        case  6:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2,  2, gpio); //LED3
        case  8:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3,  3, gpio); //LED2
        case  9: 	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 20, gpio); //SCK (fc9)
       // case 10:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  2, gpio); // I2S_TX_DATA
        case 11: 	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 22, gpio); // MISO (fc9)
       // case 12:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  3, gpio); //I2S_TX_WS
        case 13: 	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 21, gpio); // Mosi (fc9)
       // case 14:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  1, gpio); //I2S_TX_SCK
        case 15: 	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 30, gpio); // SSEL (fc9)
       // case 16:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2, 18, gpio); //I2S_RX_SCK
        case 17: 	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2,  1, gpio); //INT/PWM
       // case 18:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2, 20, gpio); //I2S_RX_WS
        case 19: 	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  7, gpio); //INT (arduino D8)
       // case 20:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2, 19, gpio); //I2S_RX_DAT
        default:    return HAL_GPIO_NOK; /* Invalid pin or not available for external access. */
        }
    }

    case LLHAL_GPIO_PORT_J10:
    {
        /*
         * J10 port: convert port and pin using MCU port and pin
         */
        switch (pin)
        {
        default:    return HAL_GPIO_NOK; /* Invalid pin or not available for external access. */
        }
    }

    case LLHAL_GPIO_PORT_J12:
    {
        /*
         * J12 port: convert port and pin using MCU port and pin
         */
        switch (pin)
        {
//        case  1:    return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  6, gpio); // SSEL (fc9)
//        case  3:    return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 16, gpio); // SPI_CS2
//        case  7:    return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 11, gpio); // MCLK_PMOD2
//        case  9:    return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 12, gpio); // CLK_OUT

        case  2:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 16, gpio); // ADC0_IN4
        case  4:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 31, gpio); // ADC0_IN5
        case  6:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_1, 0, gpio); // ADC0_IN6
        case  8:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_2, 0, gpio); // ADC0_IN7-PMOD2_INTR
        default:    return HAL_GPIO_NOK; /* Invalid pin or not available for external access. */
        }
    }

    case LLHAL_GPIO_PORT_J13:
    {
        /*
         * J13 port: convert port and pin using MCU port and pin
         */
        switch (pin)
        {
        case  1:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_1, 22, gpio); // 7 PWM
//        case  2:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0,  26, gpio); // SPIFI_CLK
//        case  4:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0,  25, gpio); // SPIFI_I01
        case  5:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 14, gpio); // 5 pwm
//        case  6:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 24, gpio); // SPIFI_I06USB0_OCCURn
        case  7:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 10, gpio); // 4 pwm
//        case  8:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 27, gpio); // SPIFI_I03
        case  9:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  5, gpio); // 3_int_ pwm
//        case  10:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 28, gpio); // SPIFI_IO2
//        case  12:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_0, 27, gpio); // SPIFI_IO3
//        case  14:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 29, gpio); // FC4_RTS_SCLX
//        case  16:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3, 28, gpio); // FC4 P1_3 PDM1_DAT
//        case  17:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3,  0, gpio); // pdm0_clk
//        case  18:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_3,  1, gpio); // FC8 P1_2 PDM1_CLK
        case  19:	return hal_lpc54608_get_gpio_data(LLHAL_GPIO_PORT_4,  4, gpio);

        default:       return HAL_GPIO_NOK; /* Invalid pin or not available for external access. */
        }
    }

    default:
        /* Invalid port. */
        return HAL_GPIO_NOK;
    }

    return HAL_GPIO_OK;
}

/* EOF -----------------------------------------------------------------------*/
