/*
 * C
 *
 * Copyright 2013-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "BESTFIT_ALLOCATOR.h"
#include <sni.h>
#include <stdio.h>
#include <string.h>
#include "bsp_util.h"

#ifdef __cplusplus
	extern "C" {
#endif

//Declare SSL memory buffer
BSP_DECLARE_BUFFER(SSL_MEM)
//SSL memory allocator
BESTFIT_ALLOCATOR SSL_allocatorInstance;
// SSL boolean to prevent a double initialization of the SSL Memory.
static int initialized = 0;

/* API -----------------------------------------------------------------------*/

void LLNET_SSL_MEM_initialize(){
	if (! initialized) {
		BESTFIT_ALLOCATOR_new(&SSL_allocatorInstance);
		BESTFIT_ALLOCATOR_initialize(&SSL_allocatorInstance, SSL_MEM_START, SSL_MEM_END);
		initialized = 1;
		#ifdef ALLOCATOR_DEBUG
			printf("LLNET_SSL_MEM_init %p - %p \n", SSL_MEM_START, SSL_MEM_END);
		#endif
	}
}

void* LLNET_SSL_MEM_allocate(int32_t size){
    void* allocationStart = BESTFIT_ALLOCATOR_allocate(&SSL_allocatorInstance, size);
#ifdef ALLOCATOR_DEBUG
	printf("LLNET_SSL_MEM_allocate %d %p \n", size, allocationStart);
#endif
    return (void*)allocationStart;
}


void LLNET_SSL_MEM_free(void* block){
	if(block){
		BESTFIT_ALLOCATOR_free(&SSL_allocatorInstance, block);
	}
#ifdef ALLOCATOR_DEBUG
    printf("LLNET_SSL_MEM_free %p \n", block);
#endif
}

#ifdef __cplusplus
	}
#endif
