/*
 * C
 *
 * Copyright 2013-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/
#include "fsl_inputmux.h"
#include "pin_mux.h"
#include "fsl_gpio.h"
#include "fsl_pint.h"
#include "microej.h"
#include "interrupts.h"
#include "LLMJVM.h"
#include "buttons_manager.h"
#include "buttons_helper.h"
#include "buttons_helper_configuration.h"

/* Definitions ---------------------------------------------------------------*/
#define PINT_PIN_INT0_SRC kINPUTMUX_GpioPort1Pin1ToPintsel

/* Global --------------------------------------------------------------------*/

static uint16_t BUTTON_PORT[NUMBER_OF_BUTTONS] = {BOARD_SW5_GPIO_PORT};
static const uint16_t BUTTON_PIN[NUMBER_OF_BUTTONS] = {BOARD_SW5_GPIO_PIN};
static const IRQn_Type BUTTON_IRQn[NUMBER_OF_BUTTONS] = {BOARD_SW5_IRQ};
static const pint_pin_int_t BUTTON_INTn[NUMBER_OF_BUTTONS] = {kPINT_PinInt1};
static const uint8_t BUTTON_ID[NUMBER_OF_BUTTONS] = {0};
static const uint8_t BUTTON_REVERSE[NUMBER_OF_BUTTONS] = {MICROEJ_TRUE};

/* Private API ---------------------------------------------------------------*/

static void BUTTONS_MANAGER_event(Button_TypeDef Button)
{
	uint8_t button_pressed;

	if (GPIO_ReadPinInput(GPIO, BUTTON_PORT[Button], BUTTON_PIN[Button]))
	{
		// GPIO == 1

		if (BUTTON_REVERSE[Button] == MICROEJ_TRUE)
		{
			// GPIO == 1 means "released"
			button_pressed = MICROEJ_FALSE;
		}
		else
		{
			// GPIO == 1 means "pressed"
			button_pressed = MICROEJ_TRUE;
			//LLMJVM_dump();
		}
	}
	else
	{
		// GPIO == 0

		if (BUTTON_REVERSE[Button] == MICROEJ_TRUE)
		{
			// GPIO == 0 means "pressed"
			button_pressed = MICROEJ_TRUE;
		}
		else
		{
			// GPIO == 0 means "released"
			button_pressed = MICROEJ_FALSE;
		}
	}

	if (button_pressed == MICROEJ_TRUE)
	{
		BUTTONS_HELPER_pressed(BUTTON_ID[Button]);
	}
	else
	{
		BUTTONS_HELPER_released(BUTTON_ID[Button]);
	}
}

/*!
* @brief Call back for PINT Pin interrupt 0.
*/
void pint_intr_button_callback(pint_pin_int_t pintr, uint32_t pmatch_status)
{
    for(uint8_t i = 0; i < NUMBER_OF_BUTTONS; i++) {
        if (pintr == BUTTON_INTn[i]) {
        	LLMJVM_dump();
            BUTTONS_MANAGER_interrupt((Button_TypeDef)i);
            break;
        }
    }
}


static void BUTTONS_MANAGER_init()
{
    BOARD_InitButtonPins();

    /* enable clock for GPIO */
    CLOCK_EnableClock(kCLOCK_Gpio1);

    /* Connect trigger sources to PINT */
    INPUTMUX_AttachSignal(INPUTMUX, BUTTON_INTn[0], PINT_PIN_INT0_SRC);

    /* Setup Pin Interrupt 0 for both rising and falling edge */
    PINT_PinInterruptConfig(PINT, BUTTON_INTn[0], kPINT_PinIntEnableBothEdges,
                            pint_intr_button_callback);

    NVIC_SetPriority(BUTTON_IRQn[0], 7);
}

/* API -----------------------------------------------------------------------*/

void BUTTONS_MANAGER_interrupt(Button_TypeDef Button)
{
    uint8_t leave = interrupt_enter();
    BUTTONS_MANAGER_event(Button);
    interrupt_leave(leave);
}

void BUTTONS_MANAGER_enable_interrupts(void)
{
	for(uint8_t i = 0; i < NUMBER_OF_BUTTONS; i++) {
        NVIC_ClearPendingIRQ(BUTTON_IRQn[i]);
        PINT_PinInterruptClrStatus(PINT, BUTTON_INTn[i]);
        EnableIRQ(BUTTON_IRQn[i]);
    }
}

void BUTTONS_MANAGER_disable_interrupts(void)
{
	for(uint8_t i = 0; i < NUMBER_OF_BUTTONS; i++) {
        DisableIRQ(BUTTON_IRQn[i]);
        PINT_PinInterruptClrStatus(PINT, BUTTON_INTn[i]);
        NVIC_ClearPendingIRQ(BUTTON_IRQn[i]);
    }
}

void BUTTONS_MANAGER_initialize(void)
{
	BUTTONS_HELPER_initialize();
	BUTTONS_MANAGER_init();
}
