
/*
 * C
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 * This file has been modified from a Freescale template.
 * This is the original template copyright:
 *
 * Copyright (c) 2013 - 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 * 
 * Copyright 2013-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "fsl_device_registers.h"
#include "fsl_debug_console.h"
#include "FreeRTOS.h"
#include "task.h"
#include "board.h"
#include "cpuload.h"

#include "pin_mux.h"
#include <stdbool.h>

#include "microjvm_main.h"

#include "SEGGER_SYSVIEW.h"
#include "SEGGER_RTT.h"

#ifdef TEST_QUALIFICATION_BUNDLE
#       include "qb_impl.h"
#endif


/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define MICROJVM_STACK_SIZE      (12 * 1024)
#define JAVA_TASK_PRIORITY       (11)                       /** Should be > tskIDLE_PRIORITY & < configTIMER_TASK_PRIORITY */
#define JAVA_TASK_STACK_SIZE     (MICROJVM_STACK_SIZE / 4)

/*******************************************************************************
 * External functions declarations
 ******************************************************************************/

/*******************************************************************************
 * Code
 ******************************************************************************/

/*!
 * @brief Java task function that runs in an OS task
 */
void xJavaTaskFunction(void * pvParameters)
{
    cpuload_init();
    //LLCOMM_stack_initialize();
      
    #ifdef TEST_QUALIFICATION_BUNDLE
    QB_IMPL_run_tests();
    #endif
        
    microjvm_main();
    vTaskDelete(xTaskGetCurrentTaskHandle());
}


/*!
 * @brief Main function
 */
int main(void)
{
  
    /* Init board hardware. */

    /* Configure main clock to max = 220 MHz. */
    // Warning if BOARD_BootClockPLL220M or BOARD_InitSDRAM try to access
    // varriable in external SDRAM it will crash because it has not been
    // initialized yet.
    BOARD_BootClockPLL220M();
    
    /* Initialized external SDRAM */
    BOARD_InitSDRAM();
    // Fill external SDRAM with zeros. This step is required by C standard.
    // If we skip it, (for example some) test for non-initialized pointer will
    // return true instead of false.
    memset((void*)(0xA0000000), 0x0, 0x800000);
    
    /* Initialize the debug console. */
    BOARD_InitDebugConsole();

    PRINTF("Perform pin initialization.\r\n");
    BOARD_InitPins();
    
    PRINTF("Initialized LCD screen.\r\n");
    BOARD_InitLCD();

    PRINTF("Enable the on-chip flash prefetch.\r\n");
    BOARD_FlashEnablePrefetch();

    PRINTF("Initialize the ENET.\r\n");
    BOARD_InitENET();

    PRINTF("Initialize the SDIF module.\r\n");
    BOARD_InitSDIF();

    PRINTF("Initialize the SPIFI module.\r\n");
    BOARD_InitSPIFI();
    

    PRINTF("Enable Segger system view\r\n");
    SEGGER_SYSVIEW_Conf();
    PRINTF("SEGGER_RTT block address: 0x%x.\r\n", &(_SEGGER_RTT));
    
    PRINTF("Board init finished.\r\n");
    
    /* Create the MicroJvm task. */
    TaskHandle_t pvCreatedTask;
    xTaskCreate(xJavaTaskFunction, "MicroJvm", JAVA_TASK_STACK_SIZE, NULL, JAVA_TASK_PRIORITY, &pvCreatedTask);
    SEGGER_SYSVIEW_setMicroJVMTask((U32)pvCreatedTask);
    vTaskStartScheduler();
}
