/*
 * C
 *
 * Copyright 2015-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp. PROPRIETARY. Use is subject to license terms.
 */

/* Includes ------------------------------------------------------------------*/

#include "time_hardware_timer.h"
#include "microej.h"
#include "fsl_ctimer.h"

// TIMER definition
#define CTIMER CTIMER3

#define CTIMER_MAT1_OUT kCTIMER_Match_1 /* Match output 1 */

#define TC_CLOCK 1000000	// clock frequency in Hertz

/* Globals -------------------------------------------------------------------*/

// this value is updated at each interrupt
static int64_t software_counter ;
static int32_t clk_freq;
static ctimer_match_config_t matchConfig1;

/* Prototypes ----------------------------------------------------------------*/

void ctimer_match1_callback(uint32_t flags);
ctimer_callback_t ctimer_callback[] = {ctimer_match1_callback};

/* Private functions ---------------------------------------------------------*/

static int64_t timer_get_counter_value(void)
{
	int64_t tc = COUNT_TO_USEC(CTIMER_GetTimerCounter(CTIMER), clk_freq);
	return tc ;
}

static void timer_set_interrupt_value(int64_t itVal)
{
	matchConfig1.matchValue = itVal;
	CTIMER_SetupMatch(CTIMER, CTIMER_MAT1_OUT, &matchConfig1);
}

/*
 * Increment the soft timer value
 */
static void timer_update_software_counter(void)
{
	int64_t currentTimerValue = timer_get_counter_value();
	software_counter += currentTimerValue ;
}

/*
 * Reconfigure interrupt value
 */
static void timer_configure_next_interrupt(void)
{
	timer_set_interrupt_value(0x7FFFFFFF);
}

/* Interrupt functions -------------------------------------------------------*/

void ctimer_match1_callback(uint32_t flags)
{
	/* Clear status flags */
	CTIMER_ClearStatusFlags(CTIMER, CTIMER_IR_MR0INT_MASK << CTIMER_MAT1_OUT);

	// increment the soft timer value and reset the hardware timer to 0
	timer_update_software_counter();
    CTIMER_Reset(CTIMER);
    
	timer_configure_next_interrupt();
}

/* Public functions ----------------------------------------------------------*/

void time_hardware_timer_initialize(void)
{
	ctimer_config_t config;

    clk_freq = CLOCK_GetFreq(kCLOCK_BusClk);

    // initialize fields
    software_counter = 0;

    CTIMER_GetDefaultConfig(&config);

    /* Init ctimer module */
    CTIMER_Init(CTIMER, &config);

	/* Configuration 1 */
	matchConfig1.enableCounterReset = false;
	matchConfig1.enableCounterStop = false;
	matchConfig1.matchValue = 0x7FFFFFFF;
	matchConfig1.outControl = kCTIMER_Output_Toggle;
	matchConfig1.outPinInitState = true;
	matchConfig1.enableInterrupt = true;

	CTIMER_RegisterCallBack(CTIMER, &ctimer_callback[0], kCTIMER_SingleCallback);
	CTIMER_SetupMatch(CTIMER, CTIMER_MAT1_OUT, &matchConfig1);
	CTIMER_StartTimer(CTIMER);
}

int64_t time_hardware_timer_getCurrentTime(void)
{
  return (software_counter + timer_get_counter_value())/1000;
}

int64_t time_hardware_timer_getTimeNanos(void)
{
	return (software_counter + timer_get_counter_value())*1000 ;
}

