/**
 * Copyright 2013-2019 MicroEJ Corp. All rights reserved.
 * For demonstration purpose only.
 * MicroEJ Corp PROPRIETARY. Use is subject to license terms.
 */
package com.is2t.microej.fp.listener;

import ej.fp.event.MicroUIButtons;
import ej.fp.event.MicroUICommand;
import ej.fp.event.MicroUIGeneric;
import ej.fp.widget.Button;
import ej.fp.widget.RepeatButton;
import ej.fp.widget.RepeatButton.RepeatButtonListener;

public class RepeatButtonListenerImpl implements RepeatButtonListener {

	/**
	 * Defines the three modes of button "USER"
	 * When the button "USER" is pressed/released...
	 *
	 * - BUTTON_SEND_BUTTON_EVENT: a MicroUI Buttons event is send (press / repeat / release)
	 * - BUTTON_SEND_COMMAND_EVENT: a MicroUI Command event is send (press / repeat only )
	 * - BUTTON_SEND_HOME_EVENT: a MicroUI Home event is send (press only)
	 *
	 * The Buttons and Command events are sent to the MicroUI application.
	 * The Home event is reserved to the internal framework.
	 */
	private static final int BUTTON_SEND_BUTTON_EVENT	= 0;
	private static final int BUTTON_SEND_COMMAND_EVENT = 1;
	private static final int BUTTON_SEND_HOME_EVENT = 2;

	/**
	 * Choose the button mode selected a value in previous list
	 */
	private static final int BUTTON_SEND_EVENT = BUTTON_SEND_HOME_EVENT;

	/**
	 * Specify which MicroUI Command to send when the mode "BUTTON_SEND_COMMAND_EVENT" is selected
	 * useful only when BUTTON_SEND_EVENT == BUTTON_SEND_COMMAND_EVENT
	 */
	private static final int BUTTON_COMMAND_EVENT = MicroUICommand.ESC;

	/**
	 * Specify the value of MicroUI Home event to send when the mode "BUTTON_SEND_HOME_EVENT" is selected
	 * useful only when BUTTON_SEND_EVENT == BUTTON_SEND_HOME_EVENT
	 */
	private static final int HOME = 1;

	@Override
	public void press(Button widget) {
		switch (BUTTON_SEND_EVENT) {
		case BUTTON_SEND_BUTTON_EVENT:
			MicroUIButtons.sendPressedEvent("BUTTONS", widget.getID());
			break;
		case BUTTON_SEND_COMMAND_EVENT:
			MicroUICommand.sendEvent("COMMANDS", BUTTON_COMMAND_EVENT);
			break;
		case BUTTON_SEND_HOME_EVENT:
			MicroUIGeneric.sendEvent("WADAPPS", HOME);
			break;
		}
	}

	@Override
	public void repeat(RepeatButton widget) {
		switch (BUTTON_SEND_EVENT) {
		case BUTTON_SEND_BUTTON_EVENT:
			MicroUIButtons.sendRepeatedEvent("BUTTONS", widget.getID());
			break;
		case BUTTON_SEND_COMMAND_EVENT:
			MicroUICommand.sendEvent("COMMANDS", BUTTON_COMMAND_EVENT);
			break;
		case BUTTON_SEND_HOME_EVENT:
			// do not send a Home event on repeat event
			break;
		}
	}

	@Override
	public void release(Button widget) {
		switch (BUTTON_SEND_EVENT) {
		case BUTTON_SEND_BUTTON_EVENT:
			MicroUIButtons.sendReleasedEvent("BUTTONS", widget.getID());
			break;
		case BUTTON_SEND_COMMAND_EVENT:
		case BUTTON_SEND_HOME_EVENT:
			// do not send a Command or Home event on release event
			break;
		}
	}
}
